#!/usr/bin/env python
# -*- coding: utf-8 -*-

#                                                           
# Copyright (C)2017 SenseDeal AI, Inc. All Rights Reserved  
#                                                           

"""                                                   
File: wechat.py
Author: lzl
E-mail: zll@sensedeal.ai
Last modified: 2019/9/18
Description:                                              
"""

import requests
import json
from .log import config, log_info


class WechatFactory(object):
    def __init__(self, label):
        self.key = config(label, 'key')
        self.url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key='
        self.headers = {'Content-Type': 'application/json'}

    def send_message(self, content: str, phone: list):
        try:
            message = {
                "msgtype": "text",
                "text": {
                    "mentioned_list": [],
                    "content": content,
                    "mentioned_mobile_list": phone
                }
            }
            _code = 0
            res = requests.post(self.url + self.key, headers=self.headers, data=json.dumps(message), timeout=10)
            if res.status_code == 200:
                return _code
        except Exception as e:
            log_info(e.__str__())
            _code = -1
        return _code




from elasticsearch_dsl import Document, Date, Integer, Keyword, Text, Float, Long
import sense_core as sd
import re
import pypinyin
import datetime
import sqlalchemy

sd.log_init_config('core', '/tmp')


class RiskStockIndex(sd.Base, sd.BaseModel):
    __tablename__ = 'risk_stock_index'
    __label__ = "mysql_stock"
    __database__ = 'stock_db'

    id = sqlalchemy.Column('id', sqlalchemy.Integer, primary_key=True, autoincrement=True)
    stock_code = sqlalchemy.Column('stock_code', sqlalchemy.String(256))
    company_name = sqlalchemy.Column('company_name', sqlalchemy.DateTime)
    combine_risk_score = sqlalchemy.Column('combine_risk_score', sqlalchemy.Float)


class StockBasicDataDocTest(Document):
    stock_code = Text(analyzer='ik_max_word')
    company_name = Text(analyzer='pinyin')
    company_name_index = Text(analyzer='ik_max_word')
    company_name_first = Text(analyzer='standard')
    risk_score = Float()
    update_time = Date()

    class Index:
        name = 'stock_basic_data_test'
        using = 'es_default'


class StockBasicDataIndexer(sd.BaseIndexer):

    def __init__(self):
        super(StockBasicDataIndexer, self).__init__('es_default', StockBasicDataDocTest)

    @classmethod
    def build_pinyin(cls, company_name):
        company_name = company_name.lower()
        pinyin = re.sub("[a-zA-Z\s+\.\!\/_,$%^*(+\"\')]+|[+——()?【】“”！，。？、~@#￥%……&*（）]+", "", company_name)
        pinyin = ''.join(cls.chinese_to_pinyin(pinyin))
        if company_name.find('st') >= 0:
            pinyin = 'st' + pinyin
        return pinyin

    @classmethod
    def chinese_to_pinyin(cls, word, first=True):
        _res = []
        for i in pypinyin.pinyin(word, style=pypinyin.NORMAL):
            if first:
                _res.extend(i[0][0])
            else:
                _res.extend(i)
        return _res

    @classmethod
    def build_doc_param(cls, stock_code, company_name, risk_score):
        return {
            'stock_code': stock_code,
            'update_time': datetime.datetime.now().strftime('%Y-%m-%d'),
            'company_name_index': company_name,
            'company_name': company_name,
            'company_name_first': cls.build_pinyin(company_name),
            'risk_score': risk_score,
        }


def test_rebuild_index():
    indexer = StockBasicDataIndexer()
    indexer.init_rebuild_index()
    index_list = RiskStockIndex.get_all()
    for index in index_list:
        param = StockBasicDataIndexer.build_doc_param(index.stock_code, index.company_name, index.combine_risk_score)
        indexer.add_bulk_data(index.stock_code, param)
    indexer.done_rebuild_index()


def test_1():
    indexer = StockBasicDataIndexer()
    indexer.init_rebuild_index()
    indexer.done_rebuild_index()






import multiprocessing
from .log import *
from .utils import *
from multiprocessing import Pool
import threadpool


def execute_threadpool(name, work_fuc, items, work_num):
    log_info("start execute_threadpool " + name)
    start = get_current_second()
    if work_num <= 1:
        for item in items:
            work_fuc(item)
        log_info("end single execute_threadpool " + name + " cost=" + str((get_current_second() - start)))
        return
    pool = threadpool.ThreadPool(work_num)
    requestss = threadpool.makeRequests(work_fuc, items)
    [pool.putRequest(req) for req in requestss]
    pool.wait()
    log_info("end execute_threadpool " + name + " cost=" + str((get_current_second() - start)))


def execute_multi_core(name, work_fuc, items, worker_num, use_multithread):
    if use_multithread:
        execute_threadpool(name, work_fuc, items, worker_num)
    else:
        executor = MultiWorkExecutor(work_fuc, items, worker_num, name)
        executor.execute()


class MultiWorkExecutor(object):

    def __init__(self, worker_fuc, jobs, work_num, name):
        self.jobs = jobs
        self.work_num = work_num
        self.worker_fuc = worker_fuc
        self.name = name

    def execute(self):
        log_info(
            "start multiprocessing handle job size=%d,work_num=%d for %s" % (len(self.jobs), self.work_num, self.name))
        start = get_current_second()
        if self.work_num <= 1 or len(self.jobs) <= self.work_num:
            for job in self.jobs:
                self.worker_fuc(job)
            return
        pool = Pool(self.work_num)  # 可以同时跑10个进程
        pool.map(self.worker_fuc, self.jobs)
        pool.close()
        pool.join()
        log_info(
            "end multiprocessing handle job size=%d,work_num=%d for %s cost=%d" % (
                len(self.jobs), self.work_num, self.name, (get_current_second() - start)))



import json
import datetime
import time
from dateutil import parser
import re
import shortuuid


def dump_json(data):
    return json.dumps(data, ensure_ascii=False)


def load_json(data):
    try:
        return json.loads(data)
    except:
        return None


def sleep(seconds):
    time.sleep(seconds)


def parse_date(str):
    return parser.parse(str)


def timestamp_to_str(time1, format='%Y-%m-%d'):
    return datetime.datetime.fromtimestamp(time1).strftime(format)


def make_timestamp(time_str=None, format='%Y-%m-%d'):
    if not time_str or time_str == 'NaT':
        time_str = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        format = '%Y-%m-%d %H:%M:%S'
    d = datetime.datetime.strptime(time_str, format)
    t = d.timetuple()
    _timestamp = int(time.mktime(t))
    return _timestamp


def timstamp_to_str(stamp, format="%Y-%m-%d %H:%M:%S"):
    return time.strftime(format, time.localtime(stamp))


def get_current_millisecond():
    return int(round(time.time() * 1000))


def get_current_second():
    return int(round(time.time()))


def now_time(format='%Y-%m-%d %H:%M:%S'):
    # 获取当前时间str
    now = datetime.datetime.now().strftime(format)
    return now


def format_time_cost(val):
    result = ''
    if val >= 3600:
        hour = int(val / 3600)
        result += str(hour) + '小时'
        val -= hour * 3600
    if val >= 60:
        min = int(val / 60)
        result += str(min) + '分'
        val -= min * 60
    result += str(val) + '秒'
    return result


def format_blank(content):
    clear_list = {
        u'\u2002': '', u'\u2003': '', u'\u2009': '', u'\u200c\u200d': '', u'\xa0': '', '&nbsp;': '',
        '&ensp;': '', '&emsp;': '', '&zwj;': '', '&zwnj;': ''
    }
    rep = dict((re.escape(k), v) for k, v in clear_list.items())
    pattern = re.compile('|'.join(rep.keys()))
    content = pattern.sub(lambda x: rep[re.escape(x.group(0))], content)
    return content


def get_url_uuid(url):
    return shortuuid.uuid(name=url)


class TimeCost(object):
    __start = get_current_millisecond()

    @classmethod
    def show_time_diff(cls):
        return format_time_cost((get_current_millisecond() - cls.__start) / 1000)

    @classmethod
    def reset_time(cls):
        cls.__start = get_current_millisecond()

    @classmethod
    def time_diff(cls):
        return get_current_millisecond() - cls.__start
